/*
 ThermostatQA - test framework for Thermostat Monitoring Tool

 Copyright 2015 Red Hat, Inc.

 This file is part of ThermostatQA

 ThermostatQA is distributed under the GNU General Public License,
 version 2 or any later version (with a special exception described
 below, commonly known as the "Classpath Exception").

 A copy of GNU General Public License (GPL) is included in this
 distribution, in the file COPYING.

 Linking ThermostatQA code with other modules is making a combined work
 based on ThermostatQA.  Thus, the terms and conditions of the GPL
 cover the whole combination.

 As a special exception, the copyright holders of ThermostatQA give you
 permission to link this code with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on ThermostatQA code.  If you modify ThermostatQA, you may
 extend this exception to your version of the software, but you are
 not obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version.
 */
package org.thermostat.qa2.tests;

import java.util.List;
import static org.thermostat.qa2.framework.Assert.*;
import org.thermostat.qa2.framework.NativeProcess;
import org.thermostat.qa2.framework.Shell;
import org.thermostat.qa2.framework.ThermostatQAConfig;
import org.thermostat.qa2.framework.annotations.*;
import org.thermostat.qa2.framework.utils.ProcessUtilities;

/**
 * Test class ported from old framework
 *
 * @author Zdeněk Žamberský
 */
@SetupStorage(type = "mongo")
@RunGnomeKeyring
@RunStorage
public class CliClientDBModificationsSmokeTest {

    public static final String[] headerPatterns = {"HOST_ID", "HOST", "VM_ID", "STATUS", "VM_NAME"};
    public static final String[] agentPatterns = {"non-existing-agent-no1", "artif.added.host0.name", "12345", "RUNNING", "unknown"};

    private static void cliDBTestCommon(boolean badLogin) throws Exception {
        ThermostatQAConfig.Login mongoLogin = ThermostatQAConfig.getMongoLogin();

        Shell mongo = ProcessUtilities.createMongoShell();
        mongo.start();
        mongo.writeln("use thermostat");
        mongo.writeln("db.auth(\"" + mongoLogin.getUsername() + "\", \"" + mongoLogin.getPassword() + "\")");
        mongo.writeln("db[\"agent-config\"].insert({ \"agentId\" : \"non-existing-agent-no1\", \"alive\" : true, \"configListenAddress\" : \"127.0.0.1:12345\", \"startTime\" : NumberLong(\"1234567891011\"), \"stopTime\" : NumberLong(0) })");
        mongo.writeln("db[\"host-info\"].insert({ \"agentId\" : \"non-existing-agent-no1\", \"hostname\" : \"artif.added.host0.name\" })");
        mongo.writeln("db[\"vm-info\"].insert({ \"agentId\" : \"non-existing-agent-no1\", \"environmentAsArray\" : [ { \"key\" : \"HOSTNAME\", \"value\" : \"artif.added.host0.name\" } ], \"startTimeStamp\" : NumberLong(\"1234567891011\"),  \"stopTimeStamp\" : NumberLong(\"-9223372036854775808\"), \"vmId\" : \"12345\", \"vmPid\" : NumberInt(\"12345\") })");
        mongo.writeln("quit()");
        mongo.waitFor();

        String[] thermostatCmds = {ThermostatQAConfig.getThermostatExecutablePath(), "shell"};
        Shell thermostat = new Shell(thermostatCmds);
        thermostat.setEnvironmentVariable("USER_THERMOSTAT_HOME", ThermostatQAConfig.getThermostatUserHome());
        thermostat.setStdOutMode(NativeProcess.MODE_LOG_AND_BUFFER);
        thermostat.start();
        thermostat.writeln("list-vms");
        if (badLogin) {
            thermostat.writeln("baduser");
            thermostat.writeln("badpassword");
        } else {
            ThermostatQAConfig.Login login = ThermostatQAConfig.getClientLogin();
            thermostat.writeln(login.getUsername());
            thermostat.writeln(login.getPassword());
        }
        thermostat.writeln("exit");
        thermostat.waitFor();
        List<String> stdoutLines = thermostat.getStdoutLines();
        if (badLogin) {
            assertNotContainsPatterns(stdoutLines, headerPatterns);
            assertNotContainsPatterns(stdoutLines, agentPatterns);
        } else {
            assertContainsPatterns(stdoutLines, headerPatterns);
            assertContainsPatterns(stdoutLines, agentPatterns);
        }
    }

    public static void cliDBTest() throws Exception {
        cliDBTestCommon(false);
    }

    public static void cliDBTestBadLogin() throws Exception {
        cliDBTestCommon(true);
    }

    @Test
    public void addVMandSeeItInShell() throws Exception {
        cliDBTest();
    }

    @Test
    public void badLogin() throws Exception {
        cliDBTestBadLogin();
    }

}
